import { shortcut } from "../../core/javascript/decorator";
import "./header.less";

/**
 * 顶部组件,提供输入框添加todo项目
 * 布局: bi.horizontal_auto 实现水平居中. bi.left_right_vertical_adapt 实现标题是输入框的靠左靠右垂直居中
 */
@shortcut()
export class ToDoListHeader extends BI.Widget {
    static xtype = "my.todolist.header";

    static EVENT_ADD = "EVENT_ADD";

    props = {
        // 指定组件的className
        baseCls: "my-todolist-header",
    }

    render() {
        const { height } = this.options;

        return {
            // 水平居中布局
            type: BI.HorizontalAutoLayout.xtype,
            items: [{
                el: {
                    // 左右垂直居中布局
                    type: BI.LeftRightVerticalAdaptLayout.xtype,
                    width: 600,
                    height: 40,
                    items: {
                        left: [
                            {
                                el: {
                                    type: BI.Label.xtype,
                                    cls: "my-todolist-title",
                                    text: "FineUI ToDoList",
                                    height,
                                },
                            },
                        ],
                        right: [
                            {
                                el: {
                                    type: BI.Editor.xtype,
                                    ref: _ref => {
                                        this.editor = _ref;
                                    },
                                    allowBlank: true,
                                    cls: "my-todolist-header-editor",
                                    watermark: "添加ToDo",
                                    width: 300,
                                    height: 24,
                                    listeners: [
                                        { // 监听bi.editor 组件的"EVENT_ENTER"事件(即敲回车),触发事件ToDoListHeader.EVENT_ADD事件并将输入框值置
                                            eventName: "EVENT_ENTER",
                                            action: () => {
                                                this.fireEvent(ToDoListHeader.EVENT_ADD, this.editor.getValue());
                                                this.editor.setValue("");
                                            },
                                        },
                                    ],
                                },
                            },
                        ],
                    },
                },
            }],
        };
    }
}
