import { Label, VerticalLayout } from 'fineui';
import { shortcut } from "../../core/typescript/decorator";
import "./list.less";

/**
 * todo项列表
 */
@shortcut()
export class List extends BI.Widget {
    public static xtype = "my.todolist.list";

    private count: Label;
    private list: VerticalLayout;

    public props = {
        // 指定组件的className
        baseCls: "my-todolist-list",
        text: "正在进行",
        items: [],
    }

    public render() {
        const { text, items } = this.options;

        return {
            type: BI.VerticalLayout.xtype,
            items: [
                {
                    el: {
                        type: BI.VerticalAdaptLayout.xtype,
                        height: 40,
                        items: [
                            {
                                type: BI.Label.xtype,
                                cls: "my-todolist-list-text",
                                textAlign: "left",
                                text,
                                width: 580,
                            }, {
                                type: BI.CenterAdaptLayout.xtype,
                                cls: "my-todolist-list-count-container",
                                width: 20,
                                height: 20,
                                items: [
                                    {
                                        el: {
                                            type: BI.Label.xtype,
                                            ref: (_ref: Label) => {
                                                this.count = _ref;
                                            },
                                            text: 0,
                                        },
                                    },
                                ],
                            },
                        ],
                    },
                }, {
                    // 用bi.vertical布局作为列表项的容器.
                    type: BI.VerticalLayout.xtype,
                    vgap: 10,
                    ref: (_ref: VerticalLayout) => {
                        this.list = _ref;
                    },
                    items: this.createItems(items),
                },
            ],
        };
    }

    private createItems(items: Item[]) {
        return items.map(item => {
            return BI.extend(item, {
                type: BI.MultiSelectItem.xtype, // 节点采用复选节点展示
                selected: item.done, // 已完成的todo项置为选中状态
                disabled: item.done, // 已完成的todo项置为灰化状态
                listeners: [
                    { // 为每个todo项添加"EVENT_CHANGE"事件监听,触发组件自身"EVENT_CHANGE"事
                        eventName: "EVENT_CHANGE",
                        action: (v: any) => {
                            this.fireEvent("EVENT_CHANGE", v);
                        },
                    },
                ],
            });
        })
    }

    private setCount(count: number) {
        this.count.setText(`${count}`);
    }

    public populate(items: []) {
        this.list.populate(this.createItems(items));
        this.setCount(items.length);
    }
}

interface Item {
    done: boolean,
}
